import base64

from apiclient import discovery
# default; set to your traffic topic. Can override on command line.
from oauth2client.client import GoogleCredentials

TRAFFIC_TOPIC = 'projects/{}/topics/{}'

PUBSUB_SCOPES = ['https://www.googleapis.com/auth/pubsub']

NUM_RETRIES = 3


class Publisher():
    _client = None
    _project = None
    _topic = None

    def __init__(self, project, topic=None, credentials=None):
        if not isinstance(project, str):
            raise Exception("Project must be a string")
        if topic and not isinstance(topic, str):
            raise Exception("topic must be a string")
        self._client = self.create_pubsub_client(credentials)
        self._project = project
        self._topic = topic

    @classmethod
    def create_pubsub_client(cls, credentials):
        """Build the pubsub client."""
        if not credentials:
            credentials = GoogleCredentials.get_application_default()
            if credentials.create_scoped_required():
                credentials = credentials.create_scoped(PUBSUB_SCOPES)
        return discovery.build('pubsub', 'v1beta2', credentials=credentials)

    def publish(cls, msg, pubsub_topic=None, msg_attributes=None):

        if cls._client is None:
            raise Exception("Class not instantiated\n"
                            "Please make sure to call Publish(project='project')")

        if not pubsub_topic and not cls._topic:
            raise Exception("Topic must be instanciated")

        if pubsub_topic:
            pubsub_topic = TRAFFIC_TOPIC.format(cls._project, pubsub_topic)
        else:
            pubsub_topic = TRAFFIC_TOPIC.format(cls._project, cls._topic)

        data = base64.b64encode(msg)
        msg_payload = {'data': data.decode("utf-8")}
        if msg_attributes:
            msg_payload['attributes'] = msg_attributes
        body = {'messages': [msg_payload]}
        resp = cls._client.projects().topics().publish(
            topic=pubsub_topic, body=body).execute(num_retries=NUM_RETRIES)
        return resp



import logging
import os
from functools import wraps

from flask import request
from google.auth.transport import requests
from google.oauth2 import id_token

HTTP_REQUEST = requests.Request()
PROJECT_ID = os.environ.get('project')


def firebase_auth(f):
    @wraps(f)
    def wrapped(*args, **kwargs):
        id_token_firebase = request.headers.get('Authorization')
        if not id_token_firebase:
            return 401, 'Unauthorized'
        try:
            claims = id_token.verify_firebase_token(id_token_firebase.split(' ').pop(), HTTP_REQUEST)
        except Exception as ex:
            logging.warning(ex.message)
            return 'Unauthorized', 401
        logging.info(claims)
        if not (claims.get('aud') == PROJECT_ID or claims.get('iss') == os.environ.get('issuer') % PROJECT_ID):
            return 'Unauthorized', 401
        kwargs["auth_user"] = {"id": claims.get('sub'), "role": claims.get('role')}
        return f(*args, **kwargs)

    return wrapped


def api_key(f):
    @wraps(f)
    def wrapped(*args, **kwargs):
        key = request.headers.get('api_key')
        logging.info(key)
        # TODO: Make an api_key restriction
        return f(*args, **kwargs)

    return wrapped
